<html>
<head>
  <title>Package</title>
</head>
<body>
{%header:Summary%}
<br>
<br>
<font size=+2>{%object_name%}</font><br>
Owner: {%owner_name%}<br>
{%sql_security%}<br>
{%object_description%} [<a
 href="fr://edit_description?parent_window={%parent_window%}&amp;object_handle={%object_handle%}&amp;object_type=PACKAGE&amp;object_name={%object_name%}">edit</a>]
<br><br>
<!--
<table cellspacing=1 cellpadding=2 border=0 bgcolor=black>
  <tbody>
    <tr bgcolor="navy">
      <td nowrap><b><font color=white>Method Type</font></b></td>
      <td nowrap><b><font color=white>Name</font></b></td>
      <td nowrap><b><font color=white>Description</font></b></td>
    </tr>
    {%foreach:method::function:
    <tr bgcolor="{%alternate:#DDDDFF:#CCCCFF%}">
      <td nowrap valign="top">{%isFunction%}</td>
      <td nowrap valign="top">{%object_name%}</td>
      <td valign="top"><font size=-1>{%object_description%} [<a
        href="fr://edit_description?parent_window={%parent_window%}&amp;object_handle={%object_handle%}&amp;object_type=COLUMN&amp;object_name={%object_name%}">edit</a>]</font></td>
    </tr>%}
  </tbody>
</table>
-->
<br><br>

<table cellspacing=1 cellpadding=3 border=0 width="98%" bgcolor="black">
<tr><td bgcolor="navy"><font color=white><b>Header</b> [<a
    href="fr://edit_package_header?parent_window={%parent_window%}&amp;object_handle={%object_handle%}"><font
    color="yellow">edit</font></a>]</font></td>
</tr>
<tr><td bgcolor="#DDDDFF"><font size=-1><pre>{%packageinfo:definition%}</pre></font></td></tr>
</table>
<br><br>
<table cellspacing=1 cellpadding=3 border=0 width="98%" bgcolor="black">
<tr><td bgcolor="navy"><font color=white><b>Body</b> [<a
    href="fr://edit_package_body?parent_window={%parent_window%}&amp;object_handle={%object_handle%}"><font
    color="yellow">edit</font></a>]</font></td>
</tr>
<tr><td bgcolor="#DDDDFF"><font size=-1><pre>{%packageinfo:source%}</pre></font></td></tr>
</table>
</body>
</html>
